#include <glut.h>
#include <math.h>
#include "SpringySheet.h"
#include "school.h"


#define BOUND                            100          //radius of box
#define SCREEN                           250          //size of viewport
#define RADIUS                           10           //radius of ball
#define NUM_VERTS			 3						//number of vertices
#define VERTEX_MASS			 1
#define FPS				 33

typedef struct VIEW_TYPE              //camera structure holds data about the camera position and angle
{
  float horizontal;
  float vertical;
  float zoom;
}VIEW;

SpringySheet* surface;
ParticleGenerator* fountain;
FishSchool* school;

VIEW camera;  					//create the camera and ball structures globally
int mouse_x, mouse_y;
bool zooming, moving;
void RenderScene(void)
{
  GLfloat lightPos[]={500.0f,500.0f,500.0f,1.0f};               //position the light at a constant location
  glLightfv(GL_LIGHT0, GL_POSITION, lightPos);                  //place light in scene
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);           //clear the screen with the background color


  glMatrixMode(GL_MODELVIEW);                                   //reset modelview matrix
  glLoadIdentity();

  glColor3f(1.0f, 0.0f, 0.0f);                                  //set color to draw cube (red)
  glTranslatef(0.0f, 0.0f, camera.zoom);                            //translate the objects away from the camera
  glRotatef(camera.horizontal, 0.0f, 1.0f, 0.0f);               //position the camera
  glRotatef(camera.vertical, 1.0f, 0.0f, 0.0f);



  glPushMatrix();							//draw the fountain
  glTranslatef(0.0, -BOUND, 0.0f);			//move to the bottom of the tank
  glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);
  glColor3f(1.0, 1.0, 0.0);					//yellow/gold
  glutSolidCone(10, 20, 10, 10);			//draw one cone
  glPopMatrix();
  glPushMatrix();
  glTranslatef(0.0, -BOUND+20, 0.0);		
  glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
  glutSolidCone(10, 20, 10, 10);			//draw the second cone
  glPopMatrix();

  school->Render();
  fountain->Render();
  surface->Render();
  fountain->Render();

  glPushMatrix();												//draw the box
  //glTranslatef(0.0f, -BOUND*0.75, 0.0f);
  glFrontFace(GL_CW);
  //glEnable(GL_CULL_FACE);
  glColor4f(0.5f, 0.5f, 1.0f, 0.2f);
  //glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  glBegin(GL_QUADS);
	glNormal3f(0,0,-1);
	glVertex3f(-BOUND, BOUND, -BOUND);
	glVertex3f(BOUND, BOUND, -BOUND);
	glVertex3f(BOUND, -BOUND, -BOUND);
	glVertex3f(-BOUND, -BOUND, -BOUND);

	glNormal3f(1, 0, 0);
	glVertex3f(BOUND, BOUND, -BOUND);
	glVertex3f(BOUND, BOUND, BOUND);
	glVertex3f(BOUND, -BOUND, BOUND);
	glVertex3f(BOUND, -BOUND, -BOUND);
	
	glNormal3f(0,0,1);
	glVertex3f(BOUND, BOUND, BOUND);
	glVertex3f(-BOUND, BOUND, BOUND);
	glVertex3f(-BOUND, -BOUND, BOUND);
	glVertex3f(BOUND, -BOUND, BOUND);

	glNormal3f(-1,0,0);
	glVertex3f(-BOUND, BOUND, BOUND);
	glVertex3f(-BOUND, BOUND, -BOUND);
	glVertex3f(-BOUND, -BOUND, -BOUND);
	glVertex3f(-BOUND, -BOUND, BOUND);
  glEnd();
	//glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glBegin(GL_QUADS);
	glNormal3f(0,-1,0);
	glVertex3f(-BOUND, -BOUND, -BOUND);
	glVertex3f(BOUND, -BOUND, -BOUND);
	glVertex3f(BOUND, -BOUND, BOUND);
	glVertex3f(-BOUND, -BOUND, BOUND);
  glEnd();
	//glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

  
  //glutSolidCube(BOUND*2);
  //glDisable(GL_CULL_FACE);

  glPopMatrix();


  

  //glColor3f(1.0f, 0.0f, 0.0f);
  //glutSolidSphere(20,20,20);

  
  glutSwapBuffers();                        //swap the background and foreground buffers for smooth animation
}


void Increment(GLint value)                      	      //increments the physics model one time step
{

	surface->Increment();
	if(surface->jet)
		fountain->Spawn(60);
	fountain->Increment();
	school->Increment(surface->vertices[7][7]->position.y, surface->jet);
	//cout<<fountain->num_particles<<endl;
  glutPostRedisplay();                         //post a redisplay message
  glutTimerFunc(FPS, Increment, 1);            //reset the timer
}

void SetupRC(void)
{
  GLfloat ambient[]={0.2f, 0.2f, 0.2f, 0.2f};                   //create light with ambient and diffuse properties
  GLfloat diffuse[]={0.8f, 0.8f, 0.8f, 0.8f};

  glClearColor(0.0f, 0.0f, 0.0f, 1.0f);                         //set the color to clear the screen to black
  glEnable(GL_DEPTH_TEST);                                      //enable depth testing
  glEnable(GL_LIGHTING);                                        //enable lighting
  glEnable(GL_COLOR_MATERIAL);                                  //attach material type to color of object
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  //glPolygonMode(GL_BACK, GL_LINE);
  //glEnable(GL_CULL_FACE);

 
  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);                    //create the light
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glEnable(GL_LIGHT0);                                          //enable it
  
  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);            //set materials to reflect ambient and diffuse light
  
}

void KeyControl(unsigned char key, GLint x, GLint y)
{
  if(key==' ')
  {
	  if(surface->jet==false)		//if the jet is off
			surface->jet=true;		//turn it on
		else
			surface->jet=false;		//otherwise, turn it off
  }
}

/*void ItemSelected(GLint value)
{
	switch(value)
	{
	case 10:
		if(surface->jet == false)
			surface->jet=true;
		else surface->jet = false;
	}
}*/

void MouseButton(int button, int state, int x, int y)
{
	if(button==GLUT_MIDDLE_BUTTON)
	{
		zooming=true; moving=false;
	}
	else
	{
		moving=true; zooming=false;
	}
	if(button==GLUT_RIGHT_BUTTON)
	{
		if(surface->jet==false)		//if the jet is off
			surface->jet=true;		//turn it on
		else
			surface->jet=false;		//otherwise, turn it off
	}
}
void MouseControl(GLint x, GLint y)
{
	if(moving)
	{
        if(x > mouse_x)
                camera.horizontal++;
        else if(x < mouse_x)
                camera.horizontal--;

        if(y > mouse_y)
                camera.vertical++;
        else if(y < mouse_y)
                camera.vertical--;
	}
	else if(zooming)
	{
		if(y > mouse_y)
			camera.zoom+=5;
		else if (y < mouse_y)
			camera.zoom-=5;
	}
	mouse_x=x;
    mouse_y=y;
}

void ChangeSize(GLsizei w, GLsizei h)                             //maintains proper aspect ratio if view is changed
{
  
  if(h==0)                                                        //prevent divide-by-zero
    h=1;
  glViewport(0,0,w,h);                                            //re-adjust the viewport
  glMatrixMode(GL_PROJECTION);                                    //reset projection matrix
  glLoadIdentity();

  gluPerspective(60.0f, (GLfloat)w/(GLfloat)h, 1.0, 1000.0);       //create a perspective direction
  glMatrixMode(GL_MODELVIEW);                                     //reset modelview matrix
  glLoadIdentity();

  
}

int main(void)
{
	camera.horizontal=camera.vertical=0.0f; camera.zoom=-300.0;//initial values of camera and ball
	mouse_x=0;
	mouse_y=0;
	moving=true;	zooming=false;
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH); //create a double buffer with RGB display
	glutCreateWindow("Dave's Gummy Cubes");					//window name
	glutDisplayFunc(RenderScene);                        //function to call to render scene
	glutReshapeFunc(ChangeSize);                         //function to call when window is redrawn
	glutTimerFunc(0, Increment, 1);                     //set timer function to maintain constant display rate
	glutKeyboardFunc(KeyControl);                         //function to call when an arrow key is pressed
	glutMotionFunc(MouseControl);
	glutMouseFunc(MouseButton);
	//glutCreateMenu(ItemSelected);
		//glutAttachMenu(GLUT_RIGHT_BUTTON);
		//glutAddMenuEntry("Jet", 10);
		//glutAddMenuEntry("Triangular Filter", 11);
		//glutAddMenuEntry("Inverse Filter", 12);
		//glutAddMenuEntry("Test Mip-Map", 13);
		//glutAddMenuEntry("Bitmap: Stripes",14);
		//glutAddMenuEntry("Bitmap: Knight", 15);


	surface=new SpringySheet(-BOUND,BOUND - 20 ,-BOUND, BOUND,BOUND-20,BOUND,1,1);
	fountain=new ParticleGenerator(0,-BOUND+20,0);
	school=new FishSchool();

	SetupRC();                                           //initialize everything
	glutMainLoop();                                      //enter the main loop
	return 0;
}