#include <glut.h>

#define MAX_PARTICLES		10000
#define MAX_AGE				60
#define SPEED				2
#define TAIL				3
#define VORTEX_TIGHTNESS	1.3
#define VORTEX_MAGNITUDE	4.0

struct point_vector
{
	double x;
	double y;
	double z;
};

struct color_vector
{
	double r;
	double g;
	double b;
};

class ParticleGenerator
{
public:
	point_vector position;
	point_vector particles[MAX_PARTICLES];
	point_vector velocities[MAX_PARTICLES];
	color_vector colors[MAX_PARTICLES];
	bool status[MAX_PARTICLES];
	int age[MAX_PARTICLES];
	int num_particles;

	ParticleGenerator(double x, double y, double z);
	void Increment();
	void Spawn(int num);
	void Render();
};

ParticleGenerator::ParticleGenerator(double x, double y, double z)
{
	position.x=x;	position.y=y;	position.z=z;

	int i;
	srand(time(NULL));
	for(i=0; i<MAX_PARTICLES; i++)
	{
		particles[i].x=x;	particles[i].y=y;	particles[i].z=z;
		colors[i].r=1.0;	colors[i].g=1.0;	colors[i].b=1.0;
		velocities[i].x=((double)rand()/RAND_MAX)-0.5;
		velocities[i].y=((double)rand()/RAND_MAX);
		velocities[i].z=((double)rand()/RAND_MAX)-0.5;

		double dot=velocities[i].x*velocities[i].x +
				   velocities[i].y*velocities[i].y +
				   velocities[i].z*velocities[i].z;
		double length=sqrt(dot);
		velocities[i].x=velocities[i].x/(length*SPEED);
		velocities[i].y=velocities[i].y/(length*SPEED);
		velocities[i].z=velocities[i].z/(length*SPEED);

		length=(double)rand()/RAND_MAX*5;					//velocity for particles
		velocities[i].x*=length;
		velocities[i].y*=length;
		velocities[i].z*=length;

		age[i]=0;
		status[i]=false;
		num_particles=0;
	}
}

void ParticleGenerator::Increment()
{
	int i;
	for(i=0; i<MAX_PARTICLES; i++)						//for every particle
	{
		if(status[i]==true)								//if the particle is active
		{
			age[i]++;									//increment it's age
			if(age[i] > MAX_AGE)
			{
				status[i]=false;	//if the particle is too old, kill it
				num_particles--;
			}
			else
			{
				particles[i].x+=velocities[i].x;			//update it's position
				particles[i].y+=velocities[i].y;
				particles[i].z+=velocities[i].z;
				//implement vortex
				double R=sqrt(particles[i].x*particles[i].x +
							  particles[i].z*particles[i].z);	//find distance from y axis
				//double theta=VORTEX_MAGNITUDE/pow(R,VORTEX_TIGHTNESS);
				double theta=3.14159/18.0;
				double x=particles[i].x;	//store the original position before rotation
				double z=particles[i].z;

				particles[i].x=x*cos(theta) - z*sin(theta);
				particles[i].z=x*sin(theta) + z*cos(theta);
			}
			
			//cout<<i<<"  "<<particles[i].x<<endl;
		}
		
	}//cout<<num_particles<<endl;


}

void ParticleGenerator::Render()
{

	int i;
	glBegin(GL_LINES);					//render the particles as lines
		for(i=0; i<MAX_PARTICLES; i++)	//for every particle
		{
			if(status[i]==true)			//if the particle is alive
			{
				glColor4f(colors[i].r,	//render the proper color
						  colors[i].g,
						  colors[i].b,
						  1.0f);
				glVertex3f(particles[i].x+velocities[i].x*TAIL,
						   particles[i].y+velocities[i].y*TAIL,
						   particles[i].z+velocities[i].z*TAIL);	//draw particles with a tail
				glColor4f(colors[i].r,	//render the proper color
						  colors[i].g,
						  colors[i].b,
						  0.0f);
				glVertex3f(particles[i].x,
						   particles[i].y,
						   particles[i].z);
				//cout<<velocities[i].x<<endl;
			}
		}
	glEnd();

	//glBegin(GL_LINES);
	//	glColor3f(1.0, 0.0, 0.0);
	//	glVertex3f(-300, -300, -300);
	//	glVertex3f(300, 300, 300);
	//glEnd();

}

void ParticleGenerator::Spawn(int num_desired)
{
	int available=MAX_PARTICLES-num_particles;
	if(available < num_desired)			//if there is not enough space
		//cout<<"not enough space"<<endl;
		return;
	int i;
	int num_produced=0;
	for(i=0; i<MAX_PARTICLES; i++)
	{
		if(status[i]==false)	//available position
		{
			status[i]=true;	//revive the particle
			age[i]=0;
			particles[i].x=position.x;
			particles[i].y=position.y;
			particles[i].z=position.z;
			num_particles++;
			num_produced++;
			//cout<<temp<<endl;
		}
		
		if(num_produced >= num_desired){return;}	//if all the particles are made, return
	}
	//test++;
	
}

