#include "generator.h"

#define NUM_FISHES		5
#define SCALE			10

#define TURN_RATE		0.05
#define FLEE_SPEED		3.0
#define SWITCH_TIME		50
#define MAX_AIR_TIME	60
#define FISHY_STRENGTH	20

#define SWIMMING		101
#define JUMPING			102
#define SCARED			103



struct fish_unit
{
	point_vector position;
	point_vector velocity;
	point_vector acceleration;
	color_vector color;
	point_vector facing;
	point_vector up;
	int status;
	int time;
	point_vector destination;
	point_vector goal;
	int air_time;
	point_vector old_facing;

};

class FishSchool
{
public:
	fish_unit fishes[NUM_FISHES];


	FishSchool();
	void Render();
	void Increment(double waterlevel, bool fountain);


};

FishSchool::FishSchool()
{
	fishes[0].position.x=0.0;	fishes[0].position.y=0.0;	fishes[0].position.z=0.0;
	fishes[0].color.r=1.0;		fishes[0].color.g=0.0;		fishes[0].color.b=0.0;

	fishes[1].position.x=50.0;	fishes[1].position.y=0.0;	fishes[1].position.z=50.0;
	fishes[1].color.r=0.0;		fishes[1].color.g=1.0;		fishes[1].color.b=0.0;

	fishes[2].position.x=0.0;	fishes[2].position.y=-50.0;	fishes[2].position.z=50.0;
	fishes[2].color.r=1.0;		fishes[2].color.g=1.0;		fishes[2].color.b=0.0;

	fishes[3].position.x=50.0;	fishes[3].position.y=-50.0;	fishes[3].position.z=50.0;
	fishes[3].color.r=1.0;		fishes[3].color.g=0.0;		fishes[3].color.b=1.0;

	fishes[4].position.x=-50.0;	fishes[4].position.y=-50.0;	fishes[4].position.z=0.0;
	fishes[4].color.r=0.0;		fishes[4].color.g=1.0;		fishes[4].color.b=1.0;

	int i;
	for(i=0; i<NUM_FISHES; i++)
	{
		fishes[i].velocity.x=fishes[i].velocity.y=fishes[i].velocity.z=(double)rand()/RAND_MAX-0.5;
		fishes[i].facing.x=1.0;	fishes[i].facing.y=0.0;	fishes[i].facing.z=0.0;
		fishes[i].up.x=0.0;	fishes[i].up.y=1.0;	fishes[i].up.z=0.0;

		
		fishes[i].status=SWIMMING;
		fishes[i].time=9999;
		fishes[i].air_time=9999;
	}

}


void FishSchool::Increment(double waterlevel, bool fountain)
{
	int i;


	
for(i=0; i<NUM_FISHES; i++)			//for each fish
{

		if(fountain && fishes[i].status !=JUMPING)	//determine the proper state for the fish
			fishes[i].status=SCARED;
		else if((waterlevel < fishes[i].position.y + FISHY_STRENGTH) &&
			    ((double)rand()/RAND_MAX < 0.2)						 &&
				(fishes[i].status != JUMPING))
		{
			fishes[i].status=JUMPING;
			fishes[i].old_facing=fishes[i].facing;			//remember the old facing
			fishes[i].velocity.y=3.0;
			fishes[i].air_time=0;
		}
		else if(fishes[i].status==JUMPING)
		{
			fishes[i].status=JUMPING;
			
			if(fishes[i].air_time > MAX_AIR_TIME)
			{
				fishes[i].status=SWIMMING;
				fishes[i].facing=fishes[i].old_facing;
			}
		}
		else
			fishes[i].status=SWIMMING;

		if(fishes[i].time > SWITCH_TIME)	//test for new goal
		{
			fishes[i].goal.x=(double)rand()/RAND_MAX * 200-100;
			fishes[i].goal.y=(double)rand()/RAND_MAX * 180-90;
			fishes[i].goal.z=(double)rand()/RAND_MAX * 200-100;
			fishes[i].time=0;
		}

		//fishes[i].status=SCARED;
		
		//if(fishes[i].status == SWIMMING)			//if the fish is swimming, go to goal
			//fishes[i].destination=fishes[i].goal;

		if(fishes[i].status == SCARED)				//if fish is scared, run away from jet
		{
								//calculate the vector directly away from the fountain
			fishes[i].destination.x=fishes[i].position.x;
			fishes[i].destination.y=fishes[i].position.y;
			fishes[i].destination.z=fishes[i].position.z;
								//and normalize it
			double length = sqrt(fishes[i].destination.x * fishes[i].destination.x +
								 fishes[i].destination.y * fishes[i].destination.y +
								 fishes[i].destination.z * fishes[i].destination.z);
			fishes[i].destination.x=(fishes[i].destination.x / length);
			fishes[i].destination.y=(fishes[i].destination.y / length);
			fishes[i].destination.z=(fishes[i].destination.z / length);
								//that is the fish's new velocity vector

			if(length > 80)	//if the fish is almost out of the tank
			{					//move it to the tangent of the circle surrounding the fountain
				point_vector u=fishes[i].destination;
				point_vector v=fishes[i].up;
				fishes[i].velocity.x= u.y*v.z - u.z*v.y;
				fishes[i].velocity.y= u.z*v.z - u.x*v.z;
				fishes[i].velocity.z= u.x*v.y - u.y*v.x;
			}
			else
			{					//else it runs away
				fishes[i].velocity=fishes[i].destination;
			}
								//properly adjust the facing
			fishes[i].facing.x=fishes[i].velocity.x;
			fishes[i].facing.z=fishes[i].velocity.z;
						
								//and the position
			fishes[i].position.x += (fishes[i].velocity.x)*FLEE_SPEED;
			fishes[i].position.y += (fishes[i].velocity.y)*0.0;
			fishes[i].position.z += (fishes[i].velocity.z)*FLEE_SPEED;



		}

		if(fishes[i].status==JUMPING)
		{
			fishes[i].velocity.y-=0.1;
			//if(waterlevel > fishes[i].position.y + FISHY_STRENGTH)
			//	fishes[i].air_time=0;
			fishes[i].air_time++;
			fishes[i].position.x+=fishes[i].velocity.x;
			fishes[i].position.y+=fishes[i].velocity.y;
			fishes[i].position.z+=fishes[i].velocity.z;

			fishes[i].facing=fishes[i].velocity;
		}


		if(fishes[i].status==SWIMMING)
		{
			fishes[i].destination=fishes[i].goal;

			if(fishes[i].destination.x < fishes[i].position.x + fishes[i].velocity.x)
				fishes[i].velocity.x -= TURN_RATE;
			else//(fishes[i].destination.x > fishes[i].position.x + fishes[i].velocity.x)
				fishes[i].velocity.x += TURN_RATE;

			if(fishes[i].destination.y < fishes[i].position.y + fishes[i].velocity.y)
				fishes[i].velocity.y -= TURN_RATE;
			else//(fishes[i].destination.y > fishes[i].position.y + fishes[i].velocity.y)
				fishes[i].velocity.y += TURN_RATE;

			if(fishes[i].destination.z < fishes[i].position.z + fishes[i].velocity.z)
				fishes[i].velocity.z -= TURN_RATE;
			else//(fishes[i].destination.z > fishes[i].position.z + fishes[i].velocity.z)
				fishes[i].velocity.z += TURN_RATE;

			double speed=sqrt(fishes[i].velocity.x*fishes[i].velocity.x +
							  fishes[i].velocity.y*fishes[i].velocity.y +
							  fishes[i].velocity.z*fishes[i].velocity.z);
			fishes[i].velocity.x = fishes[i].velocity.x /speed;
			fishes[i].velocity.y = fishes[i].velocity.y /speed;
			fishes[i].velocity.z = fishes[i].velocity.z /speed;
		

		
			fishes[i].position.x += fishes[i].velocity.x;
			fishes[i].position.y += fishes[i].velocity.y;
			fishes[i].position.z += fishes[i].velocity.z;

			fishes[i].facing.x=fishes[i].velocity.x;
			fishes[i].facing.z=fishes[i].velocity.z;

		
			fishes[i].time++;
		}

		
}
	/*speed=fishes[i].velocity.x*fishes[i].velocity.x +
					  fishes[i].velocity.y*fishes[i].velocity.y +
					  fishes[i].velocity.z*fishes[i].velocity.z;
	cout<<speed<<endl;*/
	
}

void FishSchool::Render()
{
	point_vector p0, p1, p2, p3, p4;
	int i;
	for(i=0; i<NUM_FISHES; i++)
	{
	p0.x=(fishes[i].position.x + fishes[i].facing.x*SCALE);
	p0.y=(fishes[i].position.y + fishes[i].facing.y*SCALE);
	p0.z=(fishes[i].position.z + fishes[i].facing.z*SCALE);

	p1.x=(fishes[i].position.x - fishes[i].facing.x*SCALE + fishes[i].up.x*SCALE);
	p1.y=(fishes[i].position.y - fishes[i].facing.y*SCALE + fishes[i].up.y*SCALE);
	p1.z=(fishes[i].position.z - fishes[i].facing.z*SCALE + fishes[i].up.z*SCALE);

	p3.x=(fishes[i].position.x - fishes[i].facing.x*SCALE - fishes[i].up.x*SCALE);
	p3.y=(fishes[i].position.y - fishes[i].facing.y*SCALE - fishes[i].up.y*SCALE);
	p3.z=(fishes[i].position.z - fishes[i].facing.z*SCALE - fishes[i].up.z*SCALE);

	point_vector temp, a, b;
	a=fishes[i].facing;
	b=fishes[i].up;
	temp.x=a.y*b.z - a.z*b.y;
	temp.y=a.z*b.x - a.x*b.z;
	temp.z=a.x*b.y - a.y*b.x;
	double length=sqrt(temp.x*temp.x + temp.y*temp.y + temp.z*temp.z);
	temp.x=temp.x/length;
	temp.y=temp.y/length;
	temp.z=temp.z/length;

	p4.x=(fishes[i].position.x - fishes[i].facing.x*SCALE + temp.x*SCALE);
	p4.y=(fishes[i].position.y - fishes[i].facing.y*SCALE + temp.y*SCALE);
	p4.z=(fishes[i].position.z - fishes[i].facing.z*SCALE + temp.z*SCALE);

	p2.x=(fishes[i].position.x - fishes[i].facing.x*SCALE - temp.x*SCALE);
	p2.y=(fishes[i].position.y - fishes[i].facing.y*SCALE - temp.y*SCALE);
	p2.z=(fishes[i].position.z - fishes[i].facing.z*SCALE - temp.z*SCALE);

	//glEnable(GL_CULL_FACE);
	glBegin(GL_TRIANGLE_FAN);
		glColor4f(fishes[i].color.r, fishes[i].color.g, fishes[i].color.b, 1.0f);
		glVertex3f(p0.x, p0.y, p0.z);
		glColor4f(fishes[i].color.r, fishes[i].color.g, fishes[i].color.b, 0.0f);
		//glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		glVertex3f(p1.x, p1.y, p1.z);
		glVertex3f(p2.x, p2.y, p2.z);
		glVertex3f(p3.x, p3.y, p3.z);
		glVertex3f(p4.x, p4.y, p4.z);
		glVertex3f(p1.x, p1.y, p1.z);
	glEnd();

	glBegin(GL_QUADS);
		//glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		glVertex3f(p1.x, p1.y, p1.z);
		glVertex3f(p2.x, p2.y, p2.z);
		glVertex3f(p3.x, p3.y, p3.z);
		glVertex3f(p4.x, p4.y, p4.z);
	glEnd();
	//glDisable(GL_CULL_FACE);
	}
}



	