#include "SpringyTriangle.h"

#define NUM_TRIANGLES				12
#define NUM_VERTICES				8
#define NUM_INTERNAL_EDGES 	20
#define SPRING_CONSTANT   0.09
#define DAMPER_CONSTANT   0.09
class SpringyCube
{
	private:
	SpringyVertex* v[NUM_VERTICES];							//store cube vertices
	SpringyTriangle* t[NUM_TRIANGLES];						//store cube triangles
	SpringyEdge* e[NUM_INTERNAL_EDGES];					//store internal edges not rendered
	Vector color;	
	public:
	SpringyCube(float x, float y, float z,
							float velx, float vely, float velz, float r,
							float red, float green, float blue);
	void Increment();
	void Render();
	
};

SpringyCube::SpringyCube(float x, float y, float z,
												 float velx, float vely, float velz, float r,
												 float red, float green, float blue)
{
	color.x=red;
	color.y=green;
	color.z=blue;
	v[0]=new SpringyVertex(x+r, y+r, z+r, velx, vely, velz, 1, 1.0f, 1.0f, 1.0f);		//create vertices for the cube
	v[1]=new SpringyVertex(x+r, y-r, z+r, velx, vely, velz, 1, color.x/2, color.y/2, color.z/2);
	v[2]=new SpringyVertex(x-r, y-r, z+r, velx, vely, velz, 1, color.x, color.y, color.z);
	v[3]=new SpringyVertex(x-r, y+r, z+r, velx, vely, velz, 1, color.x/2, color.y/2, color.z/2);
	v[4]=new SpringyVertex(x+r, y+r, z-r, velx, vely, velz, 1, 1.0f, 1.0f, 1.0f);
	v[5]=new SpringyVertex(x+r, y-r, z-r, velx, vely, velz, 1, color.x/2, color.y/2, color.z/2);
	v[6]=new SpringyVertex(x-r, y-r, z-r, velx, vely, velz, 1, color.x, color.y, color.z);
	v[7]=new SpringyVertex(x-r, y+r, z-r, velx, vely, velz, 1, color.x/2, color.y/2, color.z/2);
	
	t[0]=new SpringyTriangle(v[0], v[2], v[1], SPRING_CONSTANT, DAMPER_CONSTANT);
	t[1]=new SpringyTriangle(v[2], v[0], v[3], SPRING_CONSTANT, DAMPER_CONSTANT);
	t[2]=new SpringyTriangle(v[7], v[0], v[4], SPRING_CONSTANT, DAMPER_CONSTANT);
	t[3]=new SpringyTriangle(v[0], v[7], v[3], SPRING_CONSTANT, DAMPER_CONSTANT);
	t[4]=new SpringyTriangle(v[4], v[1], v[5], SPRING_CONSTANT, DAMPER_CONSTANT);
	t[5]=new SpringyTriangle(v[1], v[4], v[0], SPRING_CONSTANT, DAMPER_CONSTANT);
	t[6]=new SpringyTriangle(v[1], v[6], v[5], SPRING_CONSTANT, DAMPER_CONSTANT);
	t[7]=new SpringyTriangle(v[6], v[1], v[2], SPRING_CONSTANT, DAMPER_CONSTANT);
	t[8]=new SpringyTriangle(v[3], v[6], v[2], SPRING_CONSTANT, DAMPER_CONSTANT);
	t[9]=new SpringyTriangle(v[6], v[3], v[7], SPRING_CONSTANT, DAMPER_CONSTANT);
	t[10]=new SpringyTriangle(v[4], v[6], v[7], SPRING_CONSTANT, DAMPER_CONSTANT);
	t[11]=new SpringyTriangle(v[6], v[4], v[5], SPRING_CONSTANT, DAMPER_CONSTANT);
	
	e[0]=new SpringyEdge(v[0], v[6], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[1]=new SpringyEdge(v[6], v[0], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[2]=new SpringyEdge(v[1], v[7], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[3]=new SpringyEdge(v[7], v[1], SPRING_CONSTANT, DAMPER_CONSTANT);
	
	e[4]=new SpringyEdge(v[3], v[5], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[5]=new SpringyEdge(v[5], v[3], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[6]=new SpringyEdge(v[2], v[4], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[7]=new SpringyEdge(v[4], v[2], SPRING_CONSTANT, DAMPER_CONSTANT);
	
	e[8]=new SpringyEdge(v[0], v[5], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[9]=new SpringyEdge(v[5], v[0], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[10]=new SpringyEdge(v[3], v[6], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[11]=new SpringyEdge(v[6], v[3], SPRING_CONSTANT, DAMPER_CONSTANT);
	
	e[12]=new SpringyEdge(v[3], v[1], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[13]=new SpringyEdge(v[1], v[3], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[14]=new SpringyEdge(v[7], v[5], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[15]=new SpringyEdge(v[5], v[7], SPRING_CONSTANT, DAMPER_CONSTANT);
	
	e[16]=new SpringyEdge(v[3], v[4], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[17]=new SpringyEdge(v[4], v[3], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[18]=new SpringyEdge(v[5], v[2], SPRING_CONSTANT, DAMPER_CONSTANT);
	e[19]=new SpringyEdge(v[2], v[5], SPRING_CONSTANT, DAMPER_CONSTANT);
}

void SpringyCube::Increment()
{
	int i=0;
	
	for(i=0; i<NUM_TRIANGLES; i++)
		t[i]->Increment();
	for(i=0; i<NUM_INTERNAL_EDGES; i++)
		e[i]->Increment();
	for(i=0; i<NUM_VERTICES; i++)
		v[i]->Increment();
}

void SpringyCube::Render()
{
	int i=0;
	for(i=0; i<NUM_TRIANGLES; i++)
		t[i]->Render();
}

