#include "SpringyVertex.h"

#define TENSION		1


class SpringyVertex;


class SpringyEdge
{
	private:
	float k;		//k: spring constant
	float d;		//d: damper constant
	float l;		//l: rest length of spring
	public:
	SpringyVertex* v0;
	SpringyVertex* v1;
	SpringyEdge(SpringyVertex* a, SpringyVertex* b, float spring, float damper);
	void Increment();
};

void SpringyEdge::Increment()
{
	Vector xjxi=v1->position.Minus(v0->position);	//compute the forces acting on v0
	Vector u=xjxi; u.Normalize();
	float currentlength=xjxi.Length();
	float fk=k*(currentlength-l);
	Vector vjvi=v1->velocity.Minus(v0->velocity);
	float fd=d*(vjvi.Dot(u));
	Vector forcesk=u.Times(fk);						//forces due to the spring
	Vector forcesd=u.Times(fd);						//forces due to the damper
	v0->forces=v0->forces.Plus(forcesk.Times(TENSION));			//apply spring forces to vertex
	v0->forces=v0->forces.Plus(forcesd);			//apply damper forces to vertex

	v1->forces=v1->forces.Plus(forcesk.Times(-1*TENSION));
	v1->forces=v1->forces.Plus(forcesd.Times(-1));
}

SpringyEdge::SpringyEdge(SpringyVertex* a, SpringyVertex* b,
												 float spring, float damper)
//constructor creates a complete strut object
{
	
	v0=a;											//set variables
	v1=b;
	k=spring;
	d=damper;
	Vector edge=v0->position.Minus(v1->position);
	l=edge.Length();
	v0->AddEdge(this);								//attach this edge to the vertices
	v1->AddEdge(this);
}
	



