#include <stdlib.h>
#include <iostream.h>
#include <time.h>
#include "springyedge.h"

#define SHEET_LENGTH		15
#define SHEET_WIDTH			15

#define SPRING_C			1.0
#define DAMPER_C			1.0
#define VERTEX_M			1.0

#define DOWN_PRESSURE		0.15
#define UP_PRESSURE			0.1

class SpringySheet
{
public:
	SpringyVertex*	vertices[SHEET_LENGTH][SHEET_WIDTH];//storage space for sheet vertices
	SpringyEdge*	horizontal[SHEET_LENGTH][SHEET_WIDTH-1];//horizontal edges
	SpringyEdge*	vertical[SHEET_LENGTH-1][SHEET_WIDTH];	//vertical edges
	Vector			normals[SHEET_LENGTH][SHEET_WIDTH];
	bool jet;
public:
	SpringySheet(double x1, double y1, double z1,
						   double x2, double y2, double z2,
						   double width, double length);
	void Render();
	void Increment();
	void CalcNormals();
	void Environment();
};

SpringySheet::SpringySheet(double x1, double y1, double z1,
						   double x2, double y2, double z2,
						   double width, double length)
{
	length=SHEET_LENGTH;
	width=SHEET_WIDTH;

	//flags
	jet=false;

	//vertices = (Vector*)malloc(width*length*sizeof(Vector));//allocate memory for vertices
	double x_step=(x2-x1)/(width-1);	//calculate the step sizes for the sheet components
	double y_step=(y2-y1)/(length-1);
	double z_step=(z2-z1)/(length-1);

	double pos_x=x1;
	double pos_y=y1;
	double pos_z=z1;
	int i,j;
	//calculate vertex locations on the sheet
	for(i=0; i<SHEET_LENGTH; i++)
	{
		for(j=0; j<SHEET_WIDTH; j++)
		{
			vertices[i][j]=new SpringyVertex(pos_x, pos_y, pos_z,
											 0.0, 0.0, 0.0,
											 VERTEX_M,
											 0.0,
											 0.0,
											 1.0);
			pos_x+=x_step;
		}
		pos_x=x1;
		pos_y+=y_step;
		pos_z+=z_step;
		vertices[i][j-1]->anchor_min=0.0;
		vertices[i][j-1]->anchor_max=90.0;
	}

	for(i=0; i<SHEET_LENGTH; i++)
		for(j=0; j<SHEET_WIDTH-1; j++)
		{
			horizontal[i][j]=new SpringyEdge(vertices[i][j], vertices[i][j+1], SPRING_C, DAMPER_C);
			//cout<<"horizontal"<<i<<","<<j<<"----"<<i<<","<<j+1<<endl;
		}
	for(i=0; i<SHEET_LENGTH-1; i++)
		for(j=0; j<SHEET_WIDTH; j++)
		{
			vertical[i][j]=new SpringyEdge(vertices[i][j], vertices[i+1][j], SPRING_C, DAMPER_C);
			//cout<<"vertical"<<i<<","<<j<<"----"<<i<<","<<j+1<<endl;
		}
	for(i=0; i<SHEET_WIDTH; i++)
	{
		vertices[0][i]->anchory=true;
		vertices[SHEET_LENGTH-1][i]->anchory=true;
		vertices[i][0]->anchory=true;
		vertices[i][SHEET_WIDTH-1]->anchory=true;
	}

	vertices[0][0]->anchory=true;
	vertices[0][SHEET_WIDTH-1]->anchory=true;
	vertices[SHEET_LENGTH-1][0]->anchory=true;
	vertices[SHEET_LENGTH-1][SHEET_WIDTH-1]->anchory=true;



	for(i=0; i<SHEET_LENGTH; i++)
		for(j=0; j<SHEET_WIDTH; j++)
			normals[i][j].setCoords(0.0, 0.0, 1.0);
	CalcNormals();			//calculate the starting normals for the surface
}

void SpringySheet::CalcNormals()
{
	int i,j;



	Vector a, b;
	//calculate corner normals
	/*a=vertices[0][1]->position.Minus(vertices[0][0]->position);
	b=vertices[1][0]->position.Minus(vertices[0][0]->position);
	normals[0][0]=a.Cross(b);

	a=vertices[0][SHEET_WIDTH-1]->position.Minus(vertices[0][SHEET_WIDTH]->position);
	b=vertices[1][SHEET_WIDTH]->position.Minus(vertices[0][SHEET_WIDTH]->position);
	normals[0][SHEET_WIDTH]=a.Cross(b);

	a=vertices[SHEET_LENGTH-1][0]->position.Minus(vertices[SHEET_LENGTH][0]->position);
	b=vertices[SHEET_LENGTH][1]->position.Minus(vertices[SHEET_LENGTH][0]->position);
	normals[SHEET_LENGTH][0]=a.Cross(b);

	a=vertices[SHEET_LENGTH-1][SHEET_WIDTH]->position.Minus(vertices[SHEET_LENGTH][SHEET_WIDTH]->position);
	b=vertices[SHEET_LENGTH][SHEET_WIDTH-1]->position.Minus(vertices[SHEET_LENGTH][SHEET_WIDTH]->position);
	normals[SHEET_LENGTH][0]=a.Cross(b);*/

	//calculate internal normals
	for(i=1; i<SHEET_LENGTH-1; i++)
		for(j=1; j<SHEET_LENGTH-1; j++)
		{
			a=vertices[i-1][j]->position.Minus(vertices[i][j]->position);
			b=vertices[i][j-1]->position.Minus(vertices[i][j]->position);
			normals[i][j]=a.Cross(b);
			a=vertices[i+1][j-1]->position.Minus(vertices[i][j]->position);
			normals[i][j].Plus(b.Cross(a));
			b=vertices[i+1][j]->position.Minus(vertices[i][j]->position);
			normals[i][j].Plus(a.Cross(b));
			a=vertices[i][j+1]->position.Minus(vertices[i][j]->position);
			normals[i][j].Plus(b.Cross(a));
			b=vertices[i+1][j+1]->position.Minus(vertices[i][j]->position);
			normals[i][j].Plus(a.Cross(b));
			a=vertices[i-1][j]->position.Minus(vertices[i][j]->position);
			normals[i][j].Plus(b.Cross(a));
		}

	for(i=0; i<SHEET_LENGTH; i++)
		for(j=0; j<SHEET_WIDTH; j++)
			normals[i][j].Normalize();

}

void SpringySheet::Environment()
{
	int i,j;
	srand(time(NULL));
	//apply downward pressure
	for(i=0; i<SHEET_LENGTH; i++)
		for(j=0; j<SHEET_WIDTH; j++)
			vertices[i][j]->forces.y-=DOWN_PRESSURE* (double)rand()/RAND_MAX;
	
	//apply upward pressure
	for(i=0; i<SHEET_LENGTH; i++)
		for(j=0; j<SHEET_WIDTH; j++)
			vertices[i][j]->forces.y+=UP_PRESSURE* (double)rand()/RAND_MAX;

	if(jet)
	//for(i=SHEET_LENGTH*(1.0/2.0); i<SHEET_LENGTH*(6.0/10.0); i++)
	//	for(j=SHEET_WIDTH*(1.0/4.0); j<SHEET_WIDTH*(3.0/4.0); j++)
	//		vertices[i][j]->forces.y+=0.9;
		vertices[SHEET_LENGTH/2][SHEET_WIDTH/2]->forces.y+=10.0;
	
}






void SpringySheet::Increment()
{
	int i,j;
	Environment();

	for(i=0; i<SHEET_LENGTH; i++)				//increment horizontal edges
		for(j=0; j<SHEET_WIDTH-1; j++)
			horizontal[i][j]->Increment();

	for(i=0; i<SHEET_LENGTH-1; i++)				//increment vertical edges
		for(j=0; j<SHEET_WIDTH; j++)
			vertical[i][j]->Increment();

	for(i=0; i<SHEET_LENGTH; i++)				//increment each vertex
		for(j=0; j<SHEET_WIDTH; j++)
			vertices[i][j]->Increment();

	CalcNormals();
}

void SpringySheet::Render()
{
	glColor3f(1.0, 1.0, 1.0);
	
		for(int i=0; i<SHEET_LENGTH-1; i++)
		{
			glBegin(GL_TRIANGLE_STRIP);

			for(int j=0; j<SHEET_WIDTH; j++)
			{
				glColor4f(vertices[i][j]->color.x,
						  vertices[i][j]->color.y,
						  vertices[i][j]->color.z,
						  0.4);
				glNormal3f(normals[i][j].x,
						   normals[i][j].y,
						   normals[i][j].z);
				glVertex3f(vertices[i]  [j]->position.x,
						   vertices[i]  [j]->position.y,
						   vertices[i]  [j]->position.z);

				
				glColor4f(vertices[i+1][j]->color.x,
						  vertices[i+1][j]->color.y,
						  vertices[i+1][j]->color.z,
						  0.4);
				glNormal3f(normals[i+1][j].x,
						   normals[i+1][j].y,
						   normals[i+1][j].z);
				glVertex3f(vertices[i+1][j]->position.x,
						   vertices[i+1][j]->position.y,
						   vertices[i+1][j]->position.z);
			}
			glEnd();
		}
	glEnd();
}
