#include "SpringyEdge.h"
#include <glut.h>

class SpringyTriangle
{
	private:
	SpringyEdge* e0;																					//declare the three vertices of the triangle
	SpringyEdge* e1;
	SpringyEdge* e2;
	Vector normal;																							//surface normal for the triangle						
	void CalcNormal();
	public:
	SpringyTriangle(SpringyVertex* a, SpringyVertex* b, SpringyVertex* c, float spring, float damper)
	{
		e0=new SpringyEdge(a,b, spring, damper);
		e1=new SpringyEdge(b,c, spring, damper);
		e2=new SpringyEdge(c,a, spring, damper);
		CalcNormal();
	}
	void Increment();
	void Render();

};


void SpringyTriangle::Increment()			//increments the state of all three vertices
{

	
	e0->Increment();						//increment the state of the first vertex of every edge
	e1->Increment();
	e2->Increment();
	CalcNormal();																				//recalculate the normal
}

void SpringyTriangle::CalcNormal()
{
	Vector a=e0->v0->position.Minus(e0->v1->position);	//find two vectors representing the triangle plane
	Vector b=e1->v0->position.Minus(e1->v1->position);
	normal=a.Cross(b);																	//calculate the cross product
	normal.Normalize();																	//normalize it
}

void SpringyTriangle::Render()
{
	glBegin(GL_TRIANGLES);
		glNormal3f(normal.x, normal.y, normal.z);
		glColor3f(e0->v0->color.x, e0->v0->color.y, e0->v0->color.z);
		glVertex3f(e0->v0->position.x, e0->v0->position.y, e0->v0->position.z);
		glColor3f(e1->v0->color.x, e1->v0->color.y, e1->v0->color.z);		
		glVertex3f(e1->v0->position.x, e1->v0->position.y, e1->v0->position.z);
		glColor3f(e2->v0->color.x, e2->v0->color.y, e2->v0->color.z);
		glVertex3f(e2->v0->position.x, e2->v0->position.y, e2->v0->position.z);
	glEnd();
}

