#include "vector.h"

#define EMPTY_LIST		0	    //defines an empty list
//#define BOUND			200
#define H		     	0.24
#define COEF_FRICTION           0.5
#define GRAVITY                 0.9

class SpringyEdge;					  //be able to declare edge types

class EdgeNode						 //node used in linked list
{
	public:
	SpringyEdge* edge;
	EdgeNode* next;
	EdgeNode(SpringyEdge* newedge)
	{
		edge=newedge;
		next=EMPTY_LIST;
	}
};

class SpringyVertex
{
private:
	int num_edges;				     //number of attached edges, degree of vertex
	EdgeNode* edges;			    //linked list represents all edges attached to vertex
	float Contact_Boundary(float p1, float p2, 
                               float p3, Vector direction); //calculates elapsed time interval at a collision,places particle at edge
	bool collision;
public:
	bool anchor;
	bool anchorx, anchory, anchorz;
	double anchor_min;
	double anchor_max;
	float mass;				   //mass of the vertex
	Vector position;		     	  //vertex's current position
	Vector velocity;			 //vertex's current velocity
	Vector acceleration;			//stores the vertex's acceleration due to forces
	Vector forces;				  //used to accumulate forces from each edge
	Vector color;
	SpringyVertex(float posx, float posy, float posz,
				     float velx, float vely, float velz, float m,
				     float red, float green, float blue);
	void AddEdge(SpringyEdge* newedge);	//adds an edge to the edgelist
	void Increment();
};

SpringyVertex::SpringyVertex(float posx, float posy, float posz, //initialize vertex position and velocity
			     float velx, float vely, float velz, float m,
			     float red, float green, float blue)
{
	anchor=anchorx=anchory=anchorz=false;						//a default vertex is not an anchor
	anchor_max=80;
	anchor_min=70;
	position.x=posx;				  //initialize variables
	position.y=posy;
	position.z=posz;
	velocity.x=velx;
	velocity.y=vely;
	velocity.z=velz;
	acceleration.x=acceleration.y=acceleration.z=0;			//default to zero acceleration
	forces.x=forces.y=forces.z=0;				     //default to zero forces applied
	edges=EMPTY_LIST;
	mass=m;							  //store mass of vertex
        collision=false;
  color.x=red;
  color.y=green;
  color.z=blue;
}
	
	
void SpringyVertex::AddEdge(SpringyEdge* newedge)
{
	
	if (edges==EMPTY_LIST)					   //if the edge list is empty
	{
		edges=new EdgeNode(newedge);			 //add a new node
	}
	int i;
	EdgeNode* scan=edges;					 //create a scanning pointer
	for(i=1; i<num_edges; i++)			       //cycle to the end of the list
	{
		scan=scan->next;
	}
	scan->next=new EdgeNode(newedge);		       //create a new node at the end of the list
	num_edges++;
}

void SpringyVertex::Increment()				      //increments the state of the vertex one time step
{
	if(anchor) return;
  float x1=position.x;				      //simplify variables
  float y1=position.y;
  float z1=position.z;

  //cout<<acceleration.x<<","<<acceleration.y<<","<<acceleration.x<<endl;
  //Vector newposition=position.Plus(velocity.Times(H));
  Vector newposition = position.Plus((velocity.Times(H)));
  newposition=newposition.Plus(acceleration.Times(H*H/2.0));

  Vector tempvelocity= newposition.Minus(position);
  //Vector tempvelocity= velocity;
  float f=0.0;					      //fraction of time interval elapsed at collision
  /*if(newposition.x <= BOUND && newposition.x >= -BOUND &&
     newposition.y <= BOUND && newposition.y >= -BOUND &&
     newposition.z <= BOUND && newposition.z >= -BOUND)         //if there is no collision
    {
     f=0.0f;	 						     //no portion of the time step is used up
    }						   								
  else if(newposition.x > BOUND)                         	  //start of collision detection, one test for each side
    {
      f=Contact_Boundary(x1, BOUND, newposition.x, tempvelocity);  //place the ball on the boundry and calc remainder of time interval
      velocity.x=-velocity.x;                                    //reverse velocity
      //tempvelocity.x=-tempvelocity.x;
      collision=true;
    }
  else if(newposition.x < -BOUND)
    {
      f=Contact_Boundary(x1, -BOUND,  newposition.x, tempvelocity);
      velocity.x=-velocity.x;
      //tempvelocity.x=-tempvelocity.x;
      collision=true;
    }
   else if(newposition.y > BOUND)
     {
       f=Contact_Boundary(y1, BOUND, newposition.y, tempvelocity);
       velocity.y=-velocity.y;
       //tempvelocity.y=-tempvelocity.y;
       collision=true;
     }
   else if(newposition.y < -BOUND)
     {
       f=Contact_Boundary(y1, -BOUND, newposition.y, tempvelocity);
       velocity.y=-velocity.y;
       //tempvelocity.y=-tempvelocity.y;
       collision=true;
     }
   else if(newposition.z > BOUND)
     {
       f=Contact_Boundary(z1, BOUND, newposition.z, tempvelocity);
       velocity.z=-velocity.z;
       //tempvelocity.z=-tempvelocity.z;
       collision=true;
     }
   else if(newposition.z < -BOUND)
     {
       f=Contact_Boundary(z1, -BOUND, newposition.z, tempvelocity);
       velocity.z=-velocity.z;
       //tempvelocity.z=-tempvelocity.z;
       collision=true;
     }*/

   //alter color due to forces
	if(forces.Length() > 1.0)
	{
		color.x+=0.01;
		color.y+=0.01;
	}
	else
	{
		color.x-=0.01;
		color.y-=0.01;
	}
	if(color.x < 0)
	{
		color.x=0; color.y=0;
	}
	if(color.x > 1)
	{
		color.x=1;
		color.y=1;
	}


	if(anchory)
	{
		velocity.x=0;	velocity.z=0;
		if(position.y > anchor_max && velocity.y > 0)
			velocity.y=-velocity.y;
		if(position.y < anchor_min && velocity.y < 0)
			velocity.y=-velocity.y;
	}
	if(anchorx){velocity.y=0;	velocity.z=0;}
	if(anchorz){velocity.x=0;	velocity.y=0;}
   position=position.Plus(velocity.Times((1-f)*H));//change position of ball for remainder of time interval

   acceleration=acceleration.Times(0.0f);
   acceleration=forces.Times(1.0/mass);            //calculate acceleration based on forces and mass of the particle
   //acceleration.y-=GRAVITY;                        //apply gravitational force
   velocity=velocity.Plus(acceleration.Times(H));  //calculate velocity by adding acceleration
   if(collision==true)                             //if there was a collision
       velocity=velocity.Times(COEF_FRICTION);     //implement coefficient of friction
   collision=false;
   //acceleration=acceleration.Times(0.0f);          //clear acceleration for next time interval
   forces=forces.Times(0.0f);                      //clear acting forces for next time interval
}

float SpringyVertex::Contact_Boundary(float p1, float p2, float p3, Vector direction)
//function places the particle on the boundry of the box
//and returns the fraction of the time step that has passed
{
  float f;
  f=fabs(p2 - p3)/fabs(p1-p3);               //calculate fraction of overlap
  position.x=position.x+H*(direction.x*f);         //adjust position of ball to be at boundary
  position.y=position.y+H*(direction.y*f);
  position.z=position.z+H*(direction.z*f);
  return f;
}





